#include "RTOS_Config.h"

const unsigned long HEAP_SIZE = 2000;

sbit DATA_LED at LATD6_bit; // Mini32 leds
sbit STAT_LED at LATG6_bit; //

TOS_Task Task1;
TOS_CountingSemaphore CntSem;
WORD Cnt = 0;

//Timer1 will give the timer tick for RTOS

// Values for Mini32 board (80 Mhz CPU clock and "Peripheral Clock Divisor" Pb_Clk is Sys_Clk/1): //
// Prescaler 1:8; PR1 Preload = 10000; Actual Interrupt Time = 1 ms
void InitTimer1() {      // <--- important: procedure name NOT to be changed!
    T1CON         = 0x8010;
    T1IE_bit      = 1;
    T1IF_bit      = 0;
    T1IP0_bit     = 0;        // <--- important: interrupt priority 6 !!!!!!!!!!
    T1IP1_bit     = 1;
    T1IP2_bit     = 1;
    PR1           = 10000;
    TMR1          = 0;
}

void InitTimer2() {
    T2CON         = 0x8010;
    T2IE_bit      = 1;
    T2IF_bit      = 0;
    T2IP0_bit     = 1;
    T2IP1_bit     = 1;
    T2IP2_bit     = 1;
    PR2           = 40000;
    TMR2          = 0;
}

void Timer2Interrupt() iv IVT_TIMER_2 ilevel 7 ics ICS_SRS {
    T2IF_bit      = 0;
    //Enter your code here
    Cnt++;
    
    if (Cnt == 250) {
        OS_SignalSemaphore_ISR(CntSem); // the semaphore is signaled 3 times
        OS_SignalSemaphore_ISR(CntSem);
        OS_SignalSemaphore_ISR(CntSem);
        Cnt = 0;
    }
}

char str[6];
WORD SemVal;

void MyTask() {
    while (true) {
        OS_WaitSemaphore(CntSem);

        SemVal = OS_ReadCountingSemaphore(CntSem); // show the count value of the semaphore (after OS_Wait...)
        
        if (SemVal == 2)
            UART_Write_Text("\r\n");
            
        WordToStr(SemVal, str);
        UART_Write_Text("MyTask");
        UART_Write_Text(str);
        UART_Write_Text("\r\n");
    }
}

#pragma funcall main MyTask

void main() {
    AD1PCFG = 0xFFFF;        // Initialize AN pins as digital

    TRISD.B6 = 0;
    TRISG.B6 = 0;
    DATA_LED = 0;
    STAT_LED = 0;

    TRISE.B2 = 0;
    TRISE.B3 = 0;

    // Uart2
    UART2_Init(115200);
    Delay_ms(100);

    UART_Set_Active(UART2_Read, UART2_Write, UART2_Data_Ready, UART2_Tx_Idle);

    UART2_Write_Text("\r\n");
    UART2_Write_Text("\r\n");
    UART2_Write_Text("Start");
    UART2_Write_Text("\r\n");
    UART2_Write_Text("\r\n");

    MM_Init();
    OS_Init();

    Task1 = OS_CreateTask(MyTask);
    CntSem = OS_CreateCountingSemaphore(0);

    OS_StartTask(Task1);
    
    InitTimer2();

    EnableInterrupts();
    OS_Run();         // not in a loop!
}