#include "RTOS_Config.h"

const unsigned long HEAP_SIZE = 2000;

sbit DATA_LED at LATD6_bit; // Mini32 leds
sbit STAT_LED at LATG6_bit; //

TOS_Task Rx_Task, Tx_Task;
TOS_BinarySemaphore BufferFree;

BYTE UartBuffer[250];
WORD BufferIndex = 0;

//Timer1 will give the timer tick for RTOS

// Values for Mini32 board (80 Mhz CPU clock and "Peripheral Clock Divisor" Pb_Clk is Sys_Clk/1): //
// Prescaler 1:8; PR1 Preload = 10000; Actual Interrupt Time = 1 ms
void InitTimer1() {      // <--- important: procedure name NOT to be changed!
    T1CON         = 0x8010;
    T1IE_bit      = 1;
    T1IF_bit      = 0;
    T1IP0_bit     = 0;        // <--- important: interrupt priority 6 !!!!!!!!!!
    T1IP1_bit     = 1;
    T1IP2_bit     = 1;
    PR1           = 10000;
    TMR1          = 0;
}

void Uart_Receive_Task() {
    while (true) {
        OS_WaitSemaphore(BufferFree); // wait until buffer is free for use

        while (Uart_Data_Ready()) { // something came in
            UartBuffer[BufferIndex] = Uart_Read(); // get a byte from the Uart
            BufferIndex++;
            
            if (BufferIndex >= sizeof(UartBuffer)) 
                return;      // buffer full, stop filling it
        }

        OS_SignalSemaphore(BufferFree); // signal buffer is free for use

        OS_Yield();
    }
}

void Uart_Send_Task() {
    BYTE i;

    while (true) {
        OS_WaitSemaphore(BufferFree); // wait until buffer is free for use

        if (BufferIndex > 0) { // there is something to send
            for (i = 0; i < BufferIndex; i++) 
                UART_Write(UartBuffer[i]); // echo it
            BufferIndex = 0;
        }

        OS_SignalSemaphore(BufferFree); // signal buffer is free for use

        OS_Yield();
    }
}

#pragma funcall main Uart_Receive_Task, Uart_Send_Task

void main() {
    AD1PCFG = 0xFFFF;        // Initialize AN pins as digital

    TRISD.B6 = 0;
    TRISG.B6 = 0;
    DATA_LED = 0;
    STAT_LED = 0;

    TRISE.B2 = 0;
    TRISE.B3 = 0;

    // Uart2
    UART2_Init(115200);
    Delay_ms(100);

    UART_Set_Active(UART2_Read, UART2_Write, UART2_Data_Ready, UART2_Tx_Idle);

    UART2_Write_Text("\r\n");
    UART2_Write_Text("\r\n");
    UART2_Write_Text("Start");
    UART2_Write_Text("\r\n");
    UART2_Write_Text("\r\n");

    MM_Init();
    OS_Init();

    Rx_Task = OS_CreateTask(Uart_Receive_Task);
    Tx_Task = OS_CreateTask(Uart_Send_Task);

    BufferFree = OS_CreateBinarySemaphore(true);

    OS_StartTask(Tx_Task);
    OS_StartTask(Rx_Task);
    
    InitTimer1();

    EnableInterrupts();
    OS_Run();         // not in a loop!
}