#include "RTOS_Config.h"

const unsigned long HEAP_SIZE = 2000;

sbit DATA_LED at LATD6_bit; // Mini32 leds
sbit STAT_LED at LATG6_bit; //

char strng[21];

TOS_Task Task0_ID, Task1_ID, Task2_ID;
TOS_BinarySemaphore Semaphore, Semaphore2;

//Timer1 will give the timer tick for RTOS

// Values for Mini32 board (80 Mhz CPU clock and "Peripheral Clock Divisor" Pb_Clk is Sys_Clk/1): //
// Prescaler 1:8; PR1 Preload = 10000; Actual Interrupt Time = 1 ms
void InitTimer1() {      // <--- important: procedure name NOT to be changed!
    T1CON         = 0x8010;
    T1IE_bit      = 1;
    T1IF_bit      = 0;
    T1IP0_bit     = 0;        // <--- important: interrupt priority 6 !!!!!!!!!!
    T1IP1_bit     = 1;
    T1IP2_bit     = 1;
    PR1           = 10000;
    TMR1          = 0;
}

//Timer2
//Prescaler 1:256; PR2 Preload = 62500; Actual Interrupt Time = 200 ms

//Place/Copy this part in declaration section
void InitTimer2() {
    T2CON         = 0x8070;
    T2IE_bit      = 1;
    T2IF_bit      = 0;
    T2IP0_bit     = 1;
    T2IP1_bit     = 1;
    T2IP2_bit     = 1;
    PR2           = 62500;
    TMR2          = 0;
}

void Timer2Interrupt() iv IVT_TIMER_2 ilevel 7 ics ICS_SRS {
    T2IF_bit         = 0;
    //Enter your code here
    OS_SignalSemaphore_ISR(Semaphore2); // signal every 200 ms
}

void Task0() {
    BYTE v_;

    v_ = 0;
    while (true) {
        LATE.B2 =  ~LATE.B2;

        UART2_Write_Text("Task0 a: ");

        ByteToStr(v_, strng);
        UART2_Write_Text(strng);
        UART2_Write_Text("\r\n");


        LongWordToHex(SP, strng);
        UART2_Write_Text(strng);
        UART2_Write_Text("\r\n");

        //OS_Yield;
        OS_Delay(25);

        UART2_Write_Text("Task0 b");
        UART2_Write_Text("\r\n");


        LongWordToHex(SP, strng);
        UART2_Write_Text(strng);
        UART2_Write_Text("\r\n");


        v_++;

        OS_SignalSemaphore(Semaphore);
    }
}

void Task1() {
    BYTE v_ = 255;

    while (true) {
        OS_WaitSemaphore(Semaphore, 100);
        if (OS_TimeOut()) {
            UART2_Write_Text("TimeOut");
            UART2_Write_Text("\r\n");
        }

        LATE.B3 = ~LATE.B3;

        UART2_Write_Text("Task1 a: ");

        ByteToStr(v_, strng);
        UART2_Write_Text(strng);
        UART2_Write_Text("\r\n");


        LongWordToHex(SP, strng);
        UART2_Write_Text(strng);
        UART2_Write_Text("\r\n");


        OS_Yield();
        //OS_Delay(6);

        UART2_Write_Text("Task1 b");
        UART2_Write_Text("\r\n");


        LongWordToHex(SP, strng);
        UART2_Write_Text(strng);
        UART2_Write_Text("\r\n");

        v_--;
    }
}

void Task2() {
    while (true) {
        OS_WaitSemaphore(Semaphore2, 300);
        DATA_LED = ~DATA_LED; // payload
        UART2_Write_Text("Task2");
        UART2_Write_Text("\r\n");
    }
}

#pragma funcall main Task0, Task1, Task2

void main() {
    AD1PCFG = 0xFFFF;        // Initialize AN pins as digital

    TRISD.B6 = 0;
    TRISG.B6 = 0;
    DATA_LED = 0;
    STAT_LED = 0;

    TRISE.B2 = 0;
    TRISE.B3 = 0;

    // Uart2
    UART2_Init(115200);
    Delay_ms(100);

    //UART_Set_Active(@UART2_Read, @UART2_Write, @UART2_Data_Ready, @UART2_Tx_Idle);

    UART2_Write_Text("\r\n");
    UART2_Write_Text("\r\n");
    UART2_Write_Text("Start");
    UART2_Write_Text("\r\n");
    UART2_Write_Text("\r\n");

    MM_Init();
    OS_Init();

    LongWordToStr(SP, strng);
    UART2_Write_Text(strng);
    UART2_Write_Text("\r\n");

    Task0_ID = OS_CreateTask(Task0, 0);
    Task1_ID = OS_CreateTask(Task1, 0);
    Task2_ID = OS_CreateTask(Task2, 0);

    Semaphore  = OS_CreateBinarySemaphore(false);
    Semaphore2 = OS_CreateBinarySemaphore(false);

    OS_StartTask(Task0_ID);
    OS_StartTask(Task1_ID);
    OS_StartTask(Task2_ID);

    InitTimer2();
    EnableInterrupts();
    OS_Run();
}